/**
 * required libs:
 * SynCom.lib SynCtrl.lib websockets_static.lib Ws2_32.lib libcrypto32MD.lib libssl32MD.lib
 */

#include <winsock2.h> /* must be on top to prevent inclusion of winsock.h in windows.h */
#include <windows.h>

#include <libwebsockets.h>
#define WS_BUFFERSIZE 128
unsigned char szBuf[LWS_PRE + WS_BUFFERSIZE];
static struct lws *g_wsi;

static int callback_ws(struct lws *wsi, enum lws_callback_reasons reason, void *user, void *in, size_t len)
{
    switch (reason) {
        case LWS_CALLBACK_ESTABLISHED:
            g_wsi = wsi;
//            LogMessage("connection established\n");
//            int len = lstrlen(INDEX);
//            CopyMemory(&szBuf[LWS_SEND_BUFFER_PRE_PADDING], INDEX, len);
//            lws_write(wsi, &szBuf[LWS_SEND_BUFFER_PRE_PADDING], len, LWS_WRITE_TEXT);
            break;
        default:
            break;
    }
    return 0;
}

static struct lws_protocols protocols[] = {
    { "protocolOne", callback_ws, 0, 0 },
    { NULL, NULL, 0, 0 }
};

lws_context* ctx;

DWORD WINAPI WebsocketServerThread(LPVOID lpParam)
{
    while (1) {
        lws_service(ctx, /* timeout_ms = */ 1000000);
    }
    return 0;
}

#include "SynKit.h"

class App : public _ISynDeviceEvents
{
public:
    ISynAPI *pSynAPI;
    ISynDevice *pSynDevice;
    long x_min, x_max, y_min, y_max, z_max;

    App() {
        long lHandle = -1;
        CoInitialize(nullptr);
        if (      S_OK == CoCreateInstance(_uuidof(SynAPI), 0, CLSCTX_INPROC_SERVER, _uuidof(ISynAPI), (void**)&pSynAPI)
             && SYN_OK == pSynAPI->Initialize()
             && SYN_OK == pSynAPI->FindDevice(SE_ConnectionAny, SE_DeviceTouchPad, &lHandle)
             && SYN_OK == pSynAPI->CreateDevice(lHandle, &pSynDevice))
        {
            pSynDevice->SetSynchronousNotification(this);
            pSynDevice->GetProperty(SP_XLoSensor, &x_min);
            pSynDevice->GetProperty(SP_XHiSensor, &x_max);
            pSynDevice->GetProperty(SP_YLoSensor, &y_min);
            pSynDevice->GetProperty(SP_YHiSensor, &y_max);
            pSynDevice->GetProperty(SP_ZMaximum,  &z_max);
        }
    }

    long STDMETHODCALLTYPE OnSynDevicePacket(long lSeq) {
        SynPacket packet;
        pSynDevice->LoadPacket(packet);
        if (packet.IsMotion() || packet.IsFinger()) {
            static char buffer[128];
            double x = (double)(packet.X() - x_min) / (x_max - x_min);
            double y = (double)(packet.Y() - y_min) / (y_max - y_min);
            double z = (double) packet.Z() / z_max;
//            sprintf_s(buffer, ARRAYSIZE(buffer), "{x:%f,y:%f,z:%f}", x, y, z);
//            WriteConsole(GetStdHandle(STD_OUTPUT_HANDLE), buffer, strlen(buffer), nullptr, nullptr);

            int l = sprintf_s((char*)&szBuf[LWS_PRE], ARRAYSIZE(szBuf), R"({"x":%f,"y":%f,"z":%f})", x, y, z);
//            l = strlen(buffer);
//            CopyMemory(&szBuf[LWS_PRE], buffer, l);
//            WriteConsole(GetStdHandle(STD_OUTPUT_HANDLE), &szBuf[LWS_PRE], l, nullptr, nullptr);
            if (g_wsi) lws_write(g_wsi, &szBuf[LWS_PRE], l, LWS_WRITE_TEXT);
        }
        return 0;
    }
};

int APIENTRY WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow) {
    AllocConsole();

    App app;

    struct lws_context_creation_info info;
    ZeroMemory(&info, sizeof(info));
    info.port = 8380;
    info.protocols = protocols;
    info.gid = -1;
    info.uid = -1;
    struct lws_context* lws_ctx = lws_create_context(&info);
    if (lws_ctx == NULL) {
        return 1;
    }
    DWORD threadID;
    HANDLE thread = CreateThread(nullptr, 0, WebsocketServerThread, nullptr, 0, &threadID);

    MSG m;
    while (GetMessage(&m, nullptr, 0, 0)) DispatchMessage(&m);

    CloseHandle(thread);
    lws_context_destroy(lws_ctx);

    return 0;
}
